
// -- METHOD ----------------------------------------------------------------
/** {@inheritDoc}
@since    ${class.since}
*///-------------------------------------------------------------------------
@Override public final void assign(
  final ${class.generic.impl} pTemplate
)
{
  if ( getClass() != pTemplate.getClass() ) { // ensure to throw NullPointerException
    throw new ClassCastException( "cannot assign " + pTemplate.getClass() + " to " + getClass() );
  } else if ( pTemplate != this ) {
    doAssign( pTemplate );
  }
} // assign


// -- METHOD ----------------------------------------------------------------
/** Assigns all data members using a deep copy. Only called with a non-null
argument of the same class.
@since    ${class.since}
@param    pTemplate the data to assign
*///-------------------------------------------------------------------------
protected void doAssign(
  final ${class.generic.impl} pTemplate
)
{
  copyPrivate( pTemplate );
} // doAssign


// -- METHOD ----------------------------------------------------------------
/** {@inheritDoc}
@since    ${class.since}
*///-------------------------------------------------------------------------
@Override public final void initialize()
{
  initializeMembers( true );
} // initialize


// -- METHOD ----------------------------------------------------------------
/** Initializes all members of this instance.
@since    ${class.since}
@param    pbRecursive whether to recursively initialize created instances
*///-------------------------------------------------------------------------
public final void initialize(
  final boolean pbRecursive
)
{
  initializeMembers( pbRecursive );
} // initialize


// -- METHOD ----------------------------------------------------------------
/** Initializes all members of this instance.
@since    ${class.since}
@param    pbRecursive whether to recursively initialize created instances
*///-------------------------------------------------------------------------
protected void initializeMembers(
  final boolean pbRecursive
)
{${member.init.list}
} // initializeMembers


// -- METHOD ----------------------------------------------------------------
/** {@inheritDoc}
@since    ${class.since}
*///-------------------------------------------------------------------------
@Override public final boolean isInitialized()
{
  return areAllMembersInitialized();
} // isInitialized


// -- METHOD ----------------------------------------------------------------
/** Determines if all members of this instance are in the initialized state.
@since    ${class.since}
@return   true if all members of this instance are in the initialized state
*///-------------------------------------------------------------------------
protected boolean areAllMembersInitialized()
{
  return true${member.is.init.list};
} // areAllMembersInitialized


// -- METHOD ----------------------------------------------------------------
/** {@inheritDoc}
@since    ${class.since}
*///-------------------------------------------------------------------------
@Override public final void reset()
{
  resetMembers();
} // reset


// -- METHOD ----------------------------------------------------------------
/** Resets all members of this instance.
@since    ${class.since}
*///-------------------------------------------------------------------------
protected void resetMembers()
{${member.reset.list}
} // resetMembers


// -- METHOD ----------------------------------------------------------------
/** {@inheritDoc}
@since    ${class.since}
*///-------------------------------------------------------------------------
@Override public final boolean isReset()
{
  return areAllMembersReset();
} // isReset


// -- METHOD ----------------------------------------------------------------
/** Determines if all members of this instance are in the reset state.
@since    ${class.since}
@return   true if all members of this instance are in the reset state
*///-------------------------------------------------------------------------
protected boolean areAllMembersReset()
{
  return true${member.is.reset.list};
} // areAllMembersReset


// -- METHOD ----------------------------------------------------------------
/** {@inheritDoc}
@since    ${class.since}
*///-------------------------------------------------------------------------
@Override public final int compareTo(
  final ${class.generic.impl} pRight
)
{
  if ( this == pRight ) {
    return 0;
  } else if ( pRight == null ) {
    return 1;
  } else if ( getClass() != pRight.getClass() ) {
    return getClass().getName().compareTo( pRight.getClass().getName() );
  }
  return compareMembers( pRight );
} // compareTo


// -- METHOD ----------------------------------------------------------------
/** Compares all data members.
@since    ${class.since}
@param    pRight the object to compare to this instance
@return   a negative integer, zero, or a positive integer as this object is less
          than, equal to, or greater than the specified object
*///-------------------------------------------------------------------------
protected int compareMembers(
  final ${class.generic.impl} pRight
)
{
  int lnC = 0;
  final ${class} lRight = pRight;${member.comparison.list}
  return lnC;
} // compareMembers


// -- METHOD ----------------------------------------------------------------
/** {@inheritDoc}
@since    ${class.since}
*///-------------------------------------------------------------------------
@Override public final boolean equals(
  final Object pCompare
)
{
  if ( pCompare == this ) {
    return true;
  } else if ( pCompare == null || getClass() != pCompare.getClass() ) {
    return false;
  }
  return isEqual( pCompare );
} // equals


// -- METHOD ----------------------------------------------------------------
/** Compares the members. Only called with a non-null argument of the same class.
@since    ${class.since}
@param    pCompare the object to compare against
@return   true if equal, false otherwise
*///-------------------------------------------------------------------------
protected boolean isEqual(
  final Object pCompare
)
{
  final ${class} lCompare = ( ${class} )pCompare;
  return true${member.equals.list};
} // isEqual


// -- METHOD ----------------------------------------------------------------
/** {@inheritDoc}
@since    ${class.since}
*///-------------------------------------------------------------------------
@Override public final int hashCode()
{
  return HashTool.combine( getClass().hashCode(), computeHashCode() );
} // hashCode


// -- METHOD ----------------------------------------------------------------
/** Returns a hash code value for all data members.
@since    ${class.since}
@return   a hash code value for this objects members
*///-------------------------------------------------------------------------
protected int computeHashCode()
{
  return HashTool.combine(
    0xcafebabe${member.hashcode.list}
  );
} // computeHashCode


// -- METHOD ----------------------------------------------------------------
/** {@inheritDoc}
@since    ${class.since}
*///-------------------------------------------------------------------------
@Override public final String toString()
{
  final StringBuilder lStr = new StringBuilder( "{" );
  describeMembers( lStr );
  lStr.append( " }" );
  return lStr.toString();
} // toString


// -- METHOD ----------------------------------------------------------------
/** Visualization method for all members.
@since    ${class.since}
@param    pBuf the buffer to append to
*///-------------------------------------------------------------------------
protected void describeMembers(
  final StringBuilder pBuf
)
{${member.description.list}
} // describeMembers

